from basic_modeling_interface import Bmi
import rpy2.robjects as robjects
import numpy as np


def build_model(class_name, source_fn=None):
    robjects.r('''
        build_model = function(class, source_fn=NULL) {
            if (!is.null(source_fn)) {
                source(source_fn)
            }
            return(get(class)$new())
        }
    ''')
    return robjects.r['build_model'](class_name, source_fn)


class BmiR(Bmi):
    """Python Wrapper of a R based sub class of bmi::AbstractBmi"""
    def __init__(self, class_name, source_fn=None):
        self.model = build_model(class_name, source_fn)

    # base
    def initialize(self, filename):
        self.model['bmi_initialize'](filename)

    def update(self):
        self.model['update']()

    def update_until(self, time):
        self.model['updateUntil'](time)

    def update_fraq(self, time_frac):
        self.model['updateFrac '](time_frac)

    def finalize(self):
        self.model['bmi_finalize']()

    # info
    def get_component_name(self):
        return self.model['getComponentName']()[0]

    def get_input_var_names(self):
        return self.model['getInputVarNames']()

    def get_output_var_names(self):
        return self.model['getOutputVarNames']()

    # time
    def get_start_time(self):
        return self.model['getStartTime']()[0]

    def get_current_time(self):
        return self.model['getCurrentTime']()[0]

    def get_end_time(self):
        return self.model['getEndTime']()[0]

    def get_time_step(self):
        return self.model['getTimeStep']()[0]

    def get_time_units(self):
        return self.model['getTimeUnits']()[0]

    # vars
    def get_var_type(self, var_name):
        return self.model['getVarType'](var_name)[0]

    def get_var_units(self, var_name):
        return self.model['getVarUnits'](var_name)[0]

    def get_var_itemsize(self, var_name):
        return self.model['getVarItemSize'](var_name)[0]

    def get_var_nbytes(self, var_name):
        return self.model['getVarNBytes'](var_name)[0]

    def get_var_grid(self, var_name):
        return self.model['getVarGrid'](var_name)[0]

    # getter
    def get_value(self, var_name):
        val = self.model['getValue'](var_name)
        return np.array(val)

    def get_value_at_indices(self, var_name, indices):
        rindices = robjects.IntVector(indices)
        val = self.model['getValueAtIndices'](var_name, rindices)
        return np.array(val)

    # setter
    def set_value(self, var_name, src):
        val = np.array(src)
        if val.dtype == np.int32:
            rsrc = robjects.IntVector(val)
        else:
            rsrc = robjects.FloatVector(val)
        self.model['setValue'](var_name, rsrc)

    def set_value_at_indices(self, var_name, indices, src):
        val = np.array(src)
        if val.dtype == np.int32:
            rsrc = robjects.IntVector(val)
        else:
            rsrc = robjects.FloatVector(val)
        rindices = robjects.IntVector(indices)
        self.model['setValueAtIndices'](var_name, rindices, rsrc)

    # grid
    def get_grid_rank(self, grid_id):
        return self.model['getGridRank'](grid_id)[0]

    def get_grid_size(self, grid_id):
        return self.model['getGridSize'](grid_id)[0]

    def get_grid_type(self, grid_id):
        return self.model['getGridType'](grid_id)[0]

    def get_grid_shape(self, grid_id):
        return np.array(self.model['getGridShape'](grid_id))

    def get_grid_x(self, grid_id):
        return np.array(self.model['getGridX'](grid_id))

    def get_grid_y(self, grid_id):
        return np.array(self.model['getGridY'](grid_id))

    def get_grid_z(self, grid_id):
        return np.array(self.model['getGridZ'](grid_id))

    def get_grid_spacing(self, grid_id):
        return np.array(self.model['getGridSpacing'](grid_id))

    def get_grid_origin(self, grid_id):
        return np.array(self.model['getGridOrigin'](grid_id))

    def get_grid_connectivity(self, grid_id):
        return np.array(self.model['getGridConnectivity'](grid_id))

    def get_grid_offset(self, grid_id):
        return np.array(self.model['getGridOffset'](grid_id))



import shutil
import os
import errno
import time

import docker

from grpc4bmi.bmi_grpc_client import BmiClient


class BmiClientDocker(BmiClient):
    """
    BMI GRPC client for dockerized server processes: the initialization launches the docker container which should have the
    run-bmi-server as its command. Also, it should expose the tcp port 50001 for communication with this client. Upon
    destruction, this class terminates the corresponding docker server.
    """

    input_mount_point = "/data/input"
    output_mount_point = "/data/output"

    def __init__(self, image, image_port=50051, host=None, input_dir=None, output_dir=None, user=os.getuid()):
        port = BmiClient.get_unique_port()
        client = docker.from_env()
        volumes = {}
        self.input_dir = None
        if input_dir is not None:
            self.input_dir = os.path.abspath(input_dir)
            volumes[self.input_dir] = {"bind": BmiClientDocker.input_mount_point, "mode": "rw"}
        self.output_dir = None
        if output_dir is not None:
            self.output_dir = os.path.abspath(output_dir)
            try:
                # Create output dir ourselves, otherwise Docker will create it as root user, resulting in permission
                # errors
                os.mkdir(self.output_dir)
            except OSError as e:
                if e.errno != errno.EEXIST:
                    raise
            volumes[self.output_dir] = {"bind": BmiClientDocker.output_mount_point, "mode": "rw"}
        self.container = client.containers.run(image,
                                               ports={str(image_port) + "/tcp": port},
                                               volumes=volumes,
                                               user=user,
                                               remove=True,
                                               detach=True)
        time.sleep(1)
        super(BmiClientDocker, self).__init__(BmiClient.create_grpc_channel(port=port, host=host))

    def __del__(self):
        if hasattr(self, "container"):
            self.container.stop()

    def initialize(self, filename):
        if self.input_dir is not None:
            shutil.copy(filename, self.input_dir)
            fname = os.path.basename(filename)
            super(BmiClientDocker, self).initialize(os.path.join(BmiClientDocker.input_mount_point, fname))
        else:
            super(BmiClientDocker, self).initialize(filename)

    def get_value_ref(self, var_name):
        raise NotImplementedError("Cannot exchange memory references across process boundary")



import numpy
from heat import BmiHeat


class FlatBmiHeat(BmiHeat):
    """The BmiHeat returns multi dimensional arrays, but

    """
    def get_value(self, var_name):
        return super(FlatBmiHeat, self).get_value(var_name).flatten()

    def get_value_at_indices(self, var_name, indices):
        return super(FlatBmiHeat, self).get_value_at_indices(var_name, indices).flatten()

    def set_value(self, var_name, src):
        shape = super(FlatBmiHeat, self).get_grid_shape(super(FlatBmiHeat, self).get_var_grid(var_name))
        reshaped = numpy.reshape(src, shape)
        super(FlatBmiHeat, self).set_value(var_name, reshaped)




# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from grpc4bmi import bmi_pb2 as grpc4bmi_dot_bmi__pb2


class BmiServiceStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.initialize = channel.unary_unary(
        '/bmi.BmiService/initialize',
        request_serializer=grpc4bmi_dot_bmi__pb2.InitializeRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
        )
    self.update = channel.unary_unary(
        '/bmi.BmiService/update',
        request_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
        )
    self.updateUntil = channel.unary_unary(
        '/bmi.BmiService/updateUntil',
        request_serializer=grpc4bmi_dot_bmi__pb2.UpdateUntilRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
        )
    self.updateFrac = channel.unary_unary(
        '/bmi.BmiService/updateFrac',
        request_serializer=grpc4bmi_dot_bmi__pb2.UpdateFracRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
        )
    self.finalize = channel.unary_unary(
        '/bmi.BmiService/finalize',
        request_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
        )
    self.getComponentName = channel.unary_unary(
        '/bmi.BmiService/getComponentName',
        request_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetComponentNameResponse.FromString,
        )
    self.getInputVarNames = channel.unary_unary(
        '/bmi.BmiService/getInputVarNames',
        request_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetVarNamesResponse.FromString,
        )
    self.getOutputVarNames = channel.unary_unary(
        '/bmi.BmiService/getOutputVarNames',
        request_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetVarNamesResponse.FromString,
        )
    self.getTimeUnits = channel.unary_unary(
        '/bmi.BmiService/getTimeUnits',
        request_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetTimeUnitsResponse.FromString,
        )
    self.getTimeStep = channel.unary_unary(
        '/bmi.BmiService/getTimeStep',
        request_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetTimeStepResponse.FromString,
        )
    self.getCurrentTime = channel.unary_unary(
        '/bmi.BmiService/getCurrentTime',
        request_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetTimeResponse.FromString,
        )
    self.getStartTime = channel.unary_unary(
        '/bmi.BmiService/getStartTime',
        request_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetTimeResponse.FromString,
        )
    self.getEndTime = channel.unary_unary(
        '/bmi.BmiService/getEndTime',
        request_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetTimeResponse.FromString,
        )
    self.getVarGrid = channel.unary_unary(
        '/bmi.BmiService/getVarGrid',
        request_serializer=grpc4bmi_dot_bmi__pb2.GetVarRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetVarGridResponse.FromString,
        )
    self.getVarType = channel.unary_unary(
        '/bmi.BmiService/getVarType',
        request_serializer=grpc4bmi_dot_bmi__pb2.GetVarRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetVarTypeResponse.FromString,
        )
    self.getVarItemSize = channel.unary_unary(
        '/bmi.BmiService/getVarItemSize',
        request_serializer=grpc4bmi_dot_bmi__pb2.GetVarRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetVarItemSizeResponse.FromString,
        )
    self.getVarUnits = channel.unary_unary(
        '/bmi.BmiService/getVarUnits',
        request_serializer=grpc4bmi_dot_bmi__pb2.GetVarRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetVarUnitsResponse.FromString,
        )
    self.getVarNBytes = channel.unary_unary(
        '/bmi.BmiService/getVarNBytes',
        request_serializer=grpc4bmi_dot_bmi__pb2.GetVarRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetVarNBytesResponse.FromString,
        )
    self.getValue = channel.unary_unary(
        '/bmi.BmiService/getValue',
        request_serializer=grpc4bmi_dot_bmi__pb2.GetVarRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetValueResponse.FromString,
        )
    self.getValueAtIndices = channel.unary_unary(
        '/bmi.BmiService/getValueAtIndices',
        request_serializer=grpc4bmi_dot_bmi__pb2.GetValueAtIndicesRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetValueAtIndicesResponse.FromString,
        )
    self.setValue = channel.unary_unary(
        '/bmi.BmiService/setValue',
        request_serializer=grpc4bmi_dot_bmi__pb2.SetValueRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
        )
    self.setValueAtIndices = channel.unary_unary(
        '/bmi.BmiService/setValueAtIndices',
        request_serializer=grpc4bmi_dot_bmi__pb2.SetValueAtIndicesRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
        )
    self.getGridSize = channel.unary_unary(
        '/bmi.BmiService/getGridSize',
        request_serializer=grpc4bmi_dot_bmi__pb2.GridRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetGridSizeResponse.FromString,
        )
    self.getGridType = channel.unary_unary(
        '/bmi.BmiService/getGridType',
        request_serializer=grpc4bmi_dot_bmi__pb2.GridRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetGridTypeResponse.FromString,
        )
    self.getGridRank = channel.unary_unary(
        '/bmi.BmiService/getGridRank',
        request_serializer=grpc4bmi_dot_bmi__pb2.GridRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetGridRankResponse.FromString,
        )
    self.getGridShape = channel.unary_unary(
        '/bmi.BmiService/getGridShape',
        request_serializer=grpc4bmi_dot_bmi__pb2.GridRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetGridShapeResponse.FromString,
        )
    self.getGridSpacing = channel.unary_unary(
        '/bmi.BmiService/getGridSpacing',
        request_serializer=grpc4bmi_dot_bmi__pb2.GridRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetGridSpacingResponse.FromString,
        )
    self.getGridOrigin = channel.unary_unary(
        '/bmi.BmiService/getGridOrigin',
        request_serializer=grpc4bmi_dot_bmi__pb2.GridRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetGridOriginResponse.FromString,
        )
    self.getGridX = channel.unary_unary(
        '/bmi.BmiService/getGridX',
        request_serializer=grpc4bmi_dot_bmi__pb2.GridRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetGridPointsResponse.FromString,
        )
    self.getGridY = channel.unary_unary(
        '/bmi.BmiService/getGridY',
        request_serializer=grpc4bmi_dot_bmi__pb2.GridRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetGridPointsResponse.FromString,
        )
    self.getGridZ = channel.unary_unary(
        '/bmi.BmiService/getGridZ',
        request_serializer=grpc4bmi_dot_bmi__pb2.GridRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetGridPointsResponse.FromString,
        )
    self.getGridConnectivity = channel.unary_unary(
        '/bmi.BmiService/getGridConnectivity',
        request_serializer=grpc4bmi_dot_bmi__pb2.GridRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetGridConnectivityResponse.FromString,
        )
    self.getGridOffset = channel.unary_unary(
        '/bmi.BmiService/getGridOffset',
        request_serializer=grpc4bmi_dot_bmi__pb2.GridRequest.SerializeToString,
        response_deserializer=grpc4bmi_dot_bmi__pb2.GetGridOffsetResponse.FromString,
        )


class BmiServiceServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def initialize(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def update(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def updateUntil(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def updateFrac(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def finalize(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getComponentName(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getInputVarNames(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getOutputVarNames(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getTimeUnits(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getTimeStep(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getCurrentTime(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getStartTime(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getEndTime(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getVarGrid(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getVarType(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getVarItemSize(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getVarUnits(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getVarNBytes(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getValue(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getValueAtIndices(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def setValue(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def setValueAtIndices(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getGridSize(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getGridType(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getGridRank(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getGridShape(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getGridSpacing(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getGridOrigin(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getGridX(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getGridY(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getGridZ(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getGridConnectivity(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getGridOffset(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_BmiServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'initialize': grpc.unary_unary_rpc_method_handler(
          servicer.initialize,
          request_deserializer=grpc4bmi_dot_bmi__pb2.InitializeRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
      ),
      'update': grpc.unary_unary_rpc_method_handler(
          servicer.update,
          request_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
      ),
      'updateUntil': grpc.unary_unary_rpc_method_handler(
          servicer.updateUntil,
          request_deserializer=grpc4bmi_dot_bmi__pb2.UpdateUntilRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
      ),
      'updateFrac': grpc.unary_unary_rpc_method_handler(
          servicer.updateFrac,
          request_deserializer=grpc4bmi_dot_bmi__pb2.UpdateFracRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
      ),
      'finalize': grpc.unary_unary_rpc_method_handler(
          servicer.finalize,
          request_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
      ),
      'getComponentName': grpc.unary_unary_rpc_method_handler(
          servicer.getComponentName,
          request_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetComponentNameResponse.SerializeToString,
      ),
      'getInputVarNames': grpc.unary_unary_rpc_method_handler(
          servicer.getInputVarNames,
          request_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetVarNamesResponse.SerializeToString,
      ),
      'getOutputVarNames': grpc.unary_unary_rpc_method_handler(
          servicer.getOutputVarNames,
          request_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetVarNamesResponse.SerializeToString,
      ),
      'getTimeUnits': grpc.unary_unary_rpc_method_handler(
          servicer.getTimeUnits,
          request_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetTimeUnitsResponse.SerializeToString,
      ),
      'getTimeStep': grpc.unary_unary_rpc_method_handler(
          servicer.getTimeStep,
          request_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetTimeStepResponse.SerializeToString,
      ),
      'getCurrentTime': grpc.unary_unary_rpc_method_handler(
          servicer.getCurrentTime,
          request_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetTimeResponse.SerializeToString,
      ),
      'getStartTime': grpc.unary_unary_rpc_method_handler(
          servicer.getStartTime,
          request_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetTimeResponse.SerializeToString,
      ),
      'getEndTime': grpc.unary_unary_rpc_method_handler(
          servicer.getEndTime,
          request_deserializer=grpc4bmi_dot_bmi__pb2.Empty.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetTimeResponse.SerializeToString,
      ),
      'getVarGrid': grpc.unary_unary_rpc_method_handler(
          servicer.getVarGrid,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GetVarRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetVarGridResponse.SerializeToString,
      ),
      'getVarType': grpc.unary_unary_rpc_method_handler(
          servicer.getVarType,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GetVarRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetVarTypeResponse.SerializeToString,
      ),
      'getVarItemSize': grpc.unary_unary_rpc_method_handler(
          servicer.getVarItemSize,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GetVarRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetVarItemSizeResponse.SerializeToString,
      ),
      'getVarUnits': grpc.unary_unary_rpc_method_handler(
          servicer.getVarUnits,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GetVarRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetVarUnitsResponse.SerializeToString,
      ),
      'getVarNBytes': grpc.unary_unary_rpc_method_handler(
          servicer.getVarNBytes,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GetVarRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetVarNBytesResponse.SerializeToString,
      ),
      'getValue': grpc.unary_unary_rpc_method_handler(
          servicer.getValue,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GetVarRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetValueResponse.SerializeToString,
      ),
      'getValueAtIndices': grpc.unary_unary_rpc_method_handler(
          servicer.getValueAtIndices,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GetValueAtIndicesRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetValueAtIndicesResponse.SerializeToString,
      ),
      'setValue': grpc.unary_unary_rpc_method_handler(
          servicer.setValue,
          request_deserializer=grpc4bmi_dot_bmi__pb2.SetValueRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
      ),
      'setValueAtIndices': grpc.unary_unary_rpc_method_handler(
          servicer.setValueAtIndices,
          request_deserializer=grpc4bmi_dot_bmi__pb2.SetValueAtIndicesRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.Empty.SerializeToString,
      ),
      'getGridSize': grpc.unary_unary_rpc_method_handler(
          servicer.getGridSize,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GridRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetGridSizeResponse.SerializeToString,
      ),
      'getGridType': grpc.unary_unary_rpc_method_handler(
          servicer.getGridType,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GridRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetGridTypeResponse.SerializeToString,
      ),
      'getGridRank': grpc.unary_unary_rpc_method_handler(
          servicer.getGridRank,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GridRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetGridRankResponse.SerializeToString,
      ),
      'getGridShape': grpc.unary_unary_rpc_method_handler(
          servicer.getGridShape,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GridRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetGridShapeResponse.SerializeToString,
      ),
      'getGridSpacing': grpc.unary_unary_rpc_method_handler(
          servicer.getGridSpacing,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GridRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetGridSpacingResponse.SerializeToString,
      ),
      'getGridOrigin': grpc.unary_unary_rpc_method_handler(
          servicer.getGridOrigin,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GridRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetGridOriginResponse.SerializeToString,
      ),
      'getGridX': grpc.unary_unary_rpc_method_handler(
          servicer.getGridX,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GridRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetGridPointsResponse.SerializeToString,
      ),
      'getGridY': grpc.unary_unary_rpc_method_handler(
          servicer.getGridY,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GridRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetGridPointsResponse.SerializeToString,
      ),
      'getGridZ': grpc.unary_unary_rpc_method_handler(
          servicer.getGridZ,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GridRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetGridPointsResponse.SerializeToString,
      ),
      'getGridConnectivity': grpc.unary_unary_rpc_method_handler(
          servicer.getGridConnectivity,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GridRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetGridConnectivityResponse.SerializeToString,
      ),
      'getGridOffset': grpc.unary_unary_rpc_method_handler(
          servicer.getGridOffset,
          request_deserializer=grpc4bmi_dot_bmi__pb2.GridRequest.FromString,
          response_serializer=grpc4bmi_dot_bmi__pb2.GetGridOffsetResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'bmi.BmiService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



import os
from setuptools import setup, find_packages


# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(name="grpc4bmi",
      version="0.1.4",
      author="Gijs van den Oord",
      author_email="g.vandenoord@esciencecenter.nl",
      description="Run your BMI implementation in a separate process and expose it as BMI-python with GRPC",
      license="Apache License, Version 2.0",
      url="https://github.com/eWaterCycle/grpc4bmi",
      packages=find_packages(),
      include_package_data=True,
      long_description=read("README.md"),
      entry_points={"console_scripts": [
          "run-bmi-server =  grpc4bmi.run_server:main"
      ]},
      install_requires=[
          "grpcio",
          "grpcio-reflection",
          "protobuf",
          "numpy",
          'futures; python_version == "2.7"',
          "docker",
          "basic-modeling-interface",
      ],
      extras_require={
          'R': ['rpy2'],
      },
      classifiers=["Development Status :: 3 - Alpha",
                   "Intended Audience :: Science/Research",
                   "Programming Language :: Python",
                   "Operating System :: OS Independent",
                   "Programming Language :: Python :: 2.7",
                   "Programming Language :: Python :: 3.5",
                   "Topic :: Utilities",
                   "Topic :: Scientific/Engineering",
                   "License :: OSI Approved :: Apache Software License"
                   ],
      )
